/* 
 * Copyright © 2015 Sebastian Kügler <sebas@kde.org>
 * Copyright © 2008-2011 Kristian Høgsberg
 * Copyright © 2010-2011 Intel Corporation
 * Copyright © 2012-2013 Collabora, Ltd.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 * 
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#ifndef OUTPUTMANAGEMENT_CLIENT_PROTOCOL_H
#define OUTPUTMANAGEMENT_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_outputmanagement;
struct org_kde_kwin_outputconfiguration;

extern const struct wl_interface org_kde_kwin_outputmanagement_interface;
extern const struct wl_interface org_kde_kwin_outputconfiguration_interface;

#define ORG_KDE_KWIN_OUTPUTMANAGEMENT_CREATE_CONFIGURATION	0

static inline void
org_kde_kwin_outputmanagement_set_user_data(struct org_kde_kwin_outputmanagement *org_kde_kwin_outputmanagement, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) org_kde_kwin_outputmanagement, user_data);
}

static inline void *
org_kde_kwin_outputmanagement_get_user_data(struct org_kde_kwin_outputmanagement *org_kde_kwin_outputmanagement)
{
	return wl_proxy_get_user_data((struct wl_proxy *) org_kde_kwin_outputmanagement);
}

static inline void
org_kde_kwin_outputmanagement_destroy(struct org_kde_kwin_outputmanagement *org_kde_kwin_outputmanagement)
{
	wl_proxy_destroy((struct wl_proxy *) org_kde_kwin_outputmanagement);
}

static inline struct org_kde_kwin_outputconfiguration *
org_kde_kwin_outputmanagement_create_configuration(struct org_kde_kwin_outputmanagement *org_kde_kwin_outputmanagement)
{
	struct wl_proxy *id;

	id = wl_proxy_marshal_constructor((struct wl_proxy *) org_kde_kwin_outputmanagement,
			 ORG_KDE_KWIN_OUTPUTMANAGEMENT_CREATE_CONFIGURATION, &org_kde_kwin_outputconfiguration_interface, NULL);

	return (struct org_kde_kwin_outputconfiguration *) id;
}

/**
 * org_kde_kwin_outputconfiguration - list connected, but disabled
 *	outputs
 * @applied: configuration changes have been applied
 * @failed: configuration changes failed to apply
 *
 * outputconfiguration is a client-specific resource that can be used to
 * ask the compositor to apply changes to the outputdevices, the monitors
 * connected to the system. The client receives a list of outputdevices
 * from the registry. When it wants to apply new settings, it creates a
 * configuration object from the outputmanagement global, writes changes
 * through this object's enable, scale, transform and mode calls. It then
 * asks the compositor to apply these settings in an atomic fashion, for
 * example through Linux's DRM interface. The compositor signals back
 * whether the new settings have applied successfully or failed to apply.
 * outputdevices are updated after the hardware changes have been applied
 * and before the server side fires applied.
 */
struct org_kde_kwin_outputconfiguration_listener {
	/**
	 * applied - configuration changes have been applied
	 *
	 * Fired after the compositor has applied the changes
	 * successfully.
	 */
	void (*applied)(void *data,
			struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration);
	/**
	 * failed - configuration changes failed to apply
	 *
	 * Fired if the compositor rejects or failed to apply the
	 * changes.
	 */
	void (*failed)(void *data,
		       struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration);
};

static inline int
org_kde_kwin_outputconfiguration_add_listener(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration,
					      const struct org_kde_kwin_outputconfiguration_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) org_kde_kwin_outputconfiguration,
				     (void (**)(void)) listener, data);
}

#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_ENABLE	0
#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_MODE	1
#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_TRANSFORM	2
#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_POSITION	3
#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_SCALE	4
#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_APPLY	5

static inline void
org_kde_kwin_outputconfiguration_set_user_data(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) org_kde_kwin_outputconfiguration, user_data);
}

static inline void *
org_kde_kwin_outputconfiguration_get_user_data(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration)
{
	return wl_proxy_get_user_data((struct wl_proxy *) org_kde_kwin_outputconfiguration);
}

static inline void
org_kde_kwin_outputconfiguration_destroy(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration)
{
	wl_proxy_destroy((struct wl_proxy *) org_kde_kwin_outputconfiguration);
}

static inline void
org_kde_kwin_outputconfiguration_enable(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration, struct org_kde_kwin_outputdevice *outputdevice, int32_t enable)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_outputconfiguration,
			 ORG_KDE_KWIN_OUTPUTCONFIGURATION_ENABLE, outputdevice, enable);
}

static inline void
org_kde_kwin_outputconfiguration_mode(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration, struct org_kde_kwin_outputdevice *outputdevice, int32_t mode_id)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_outputconfiguration,
			 ORG_KDE_KWIN_OUTPUTCONFIGURATION_MODE, outputdevice, mode_id);
}

static inline void
org_kde_kwin_outputconfiguration_transform(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration, struct org_kde_kwin_outputdevice *outputdevice, int32_t transform)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_outputconfiguration,
			 ORG_KDE_KWIN_OUTPUTCONFIGURATION_TRANSFORM, outputdevice, transform);
}

static inline void
org_kde_kwin_outputconfiguration_position(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration, struct org_kde_kwin_outputdevice *outputdevice, int32_t x, int32_t y)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_outputconfiguration,
			 ORG_KDE_KWIN_OUTPUTCONFIGURATION_POSITION, outputdevice, x, y);
}

static inline void
org_kde_kwin_outputconfiguration_scale(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration, struct org_kde_kwin_outputdevice *outputdevice, int32_t scale)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_outputconfiguration,
			 ORG_KDE_KWIN_OUTPUTCONFIGURATION_SCALE, outputdevice, scale);
}

static inline void
org_kde_kwin_outputconfiguration_apply(struct org_kde_kwin_outputconfiguration *org_kde_kwin_outputconfiguration)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_outputconfiguration,
			 ORG_KDE_KWIN_OUTPUTCONFIGURATION_APPLY);
}

#ifdef  __cplusplus
}
#endif

#endif
