/* 
 * Copyright (C) 2015 Martin Gräßlin
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DPMS_SERVER_PROTOCOL_H
#define DPMS_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_dpms_manager;
struct org_kde_kwin_dpms;

extern const struct wl_interface org_kde_kwin_dpms_manager_interface;
extern const struct wl_interface org_kde_kwin_dpms_interface;

/**
 * org_kde_kwin_dpms_manager - Output dpms manager
 * @get: Get org_kde_kwin_dpms for wl_output
 *
 * The Dpms manager allows to get a org_kde_kwin_dpms for a given
 * wl_output. The org_kde_kwin_dpms provides the currently used VESA
 * Display Power Management Signaling state (see
 * https://en.wikipedia.org/wiki/VESA_Display_Power_Management_Signaling ).
 * In addition it allows to request a state change. A compositor is not
 * obliged to honor it and will normally automatically switch back to on
 * state.
 */
struct org_kde_kwin_dpms_manager_interface {
	/**
	 * get - Get org_kde_kwin_dpms for wl_output
	 * @id: (none)
	 * @output: (none)
	 *
	 * Factory request to get the org_kde_kwin_dpms for a given
	 * wl_output.
	 */
	void (*get)(struct wl_client *client,
		    struct wl_resource *resource,
		    uint32_t id,
		    struct wl_resource *output);
};


#ifndef ORG_KDE_KWIN_DPMS_MODE_ENUM
#define ORG_KDE_KWIN_DPMS_MODE_ENUM
enum org_kde_kwin_dpms_mode {
	ORG_KDE_KWIN_DPMS_MODE_ON = 0,
	ORG_KDE_KWIN_DPMS_MODE_STANDBY = 1,
	ORG_KDE_KWIN_DPMS_MODE_SUSPEND = 2,
	ORG_KDE_KWIN_DPMS_MODE_OFF = 3,
};
#endif /* ORG_KDE_KWIN_DPMS_MODE_ENUM */

/**
 * org_kde_kwin_dpms - Dpms for a wl_output
 * @set: Request DPMS state change for the wl_output
 * @release: release the dpms object
 *
 * This interface provides information about the VESA DPMS state for a
 * wl_output. It gets created through the request get on the
 * org_kde_kwin_dpms_manager interface.
 *
 * On creating the resource the server will push whether DPSM is supported
 * for the output, the currently used DPMS state and notifies the client
 * through the done event once all states are pushed. Whenever a state
 * changes the set of changes is committed with the done event.
 */
struct org_kde_kwin_dpms_interface {
	/**
	 * set - Request DPMS state change for the wl_output
	 * @mode: Requested mode
	 *
	 * Requests that the compositor puts the wl_output into the
	 * passed mode. The compositor is not obliged to change the state.
	 * In addition the compositor might leave the mode whenever it
	 * seems suitable. E.g. the compositor might return to On state on
	 * user input.
	 *
	 * The client should not assume that the mode changed after
	 * requesting a new mode. Instead the client should listen for the
	 * mode event.
	 */
	void (*set)(struct wl_client *client,
		    struct wl_resource *resource,
		    uint32_t mode);
	/**
	 * release - release the dpms object
	 *
	 * 
	 */
	void (*release)(struct wl_client *client,
			struct wl_resource *resource);
};

#define ORG_KDE_KWIN_DPMS_SUPPORTED	0
#define ORG_KDE_KWIN_DPMS_MODE	1
#define ORG_KDE_KWIN_DPMS_DONE	2

#define ORG_KDE_KWIN_DPMS_SUPPORTED_SINCE_VERSION	1
#define ORG_KDE_KWIN_DPMS_MODE_SINCE_VERSION	1
#define ORG_KDE_KWIN_DPMS_DONE_SINCE_VERSION	1

static inline void
org_kde_kwin_dpms_send_supported(struct wl_resource *resource_, uint32_t supported)
{
	wl_resource_post_event(resource_, ORG_KDE_KWIN_DPMS_SUPPORTED, supported);
}

static inline void
org_kde_kwin_dpms_send_mode(struct wl_resource *resource_, uint32_t mode)
{
	wl_resource_post_event(resource_, ORG_KDE_KWIN_DPMS_MODE, mode);
}

static inline void
org_kde_kwin_dpms_send_done(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ORG_KDE_KWIN_DPMS_DONE);
}

#ifdef  __cplusplus
}
#endif

#endif
