/* 
 * Copyright (C) 2013-2014 Pier Luigi Fiorini
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PLASMA_WINDOW_MANAGEMENT_SERVER_PROTOCOL_H
#define PLASMA_WINDOW_MANAGEMENT_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct org_kde_plasma_window_management;
struct org_kde_plasma_window;

extern const struct wl_interface org_kde_plasma_window_management_interface;
extern const struct wl_interface org_kde_plasma_window_interface;

#ifndef ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ENUM
#define ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ENUM
enum org_kde_plasma_window_management_state {
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ACTIVE = 1 << 0,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZED = 1 << 1,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZED = 1 << 2,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREEN = 1 << 3,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_ABOVE = 1 << 4,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_BELOW = 1 << 5,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ON_ALL_DESKTOPS = 1 << 6,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_DEMANDS_ATTENTION = 1 << 7,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_CLOSEABLE = 1 << 8,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZABLE = 1 << 9,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZABLE = 1 << 10,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREENABLE = 1 << 11,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR = 1 << 12,
};
#endif /* ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ENUM */

#ifndef ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_ENUM
#define ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_ENUM
enum org_kde_plasma_window_management_show_desktop {
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_DISABLED = 0,
	ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_ENABLED = 1,
};
#endif /* ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_ENUM */

/**
 * org_kde_plasma_window_management - application windows management
 * @show_desktop: show/hide the desktop
 * @get_window: (none)
 *
 * This interface manages application windows. It provides requests to
 * show and hide the desktop and emits an event every time a window is
 * created so that the client can use it to manage the window.
 *
 * Only one client can bind this interface at a time.
 */
struct org_kde_plasma_window_management_interface {
	/**
	 * show_desktop - show/hide the desktop
	 * @state: requested state
	 *
	 * Tell the compositor to show/hide the desktop.
	 */
	void (*show_desktop)(struct wl_client *client,
			     struct wl_resource *resource,
			     uint32_t state);
	/**
	 * get_window - (none)
	 * @id: (none)
	 * @internal_window_id: The internal window id of the window to
	 *	create
	 */
	void (*get_window)(struct wl_client *client,
			   struct wl_resource *resource,
			   uint32_t id,
			   uint32_t internal_window_id);
};

#define ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_CHANGED	0
#define ORG_KDE_PLASMA_WINDOW_MANAGEMENT_WINDOW	1

#define ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_CHANGED_SINCE_VERSION	1
#define ORG_KDE_PLASMA_WINDOW_MANAGEMENT_WINDOW_SINCE_VERSION	1

static inline void
org_kde_plasma_window_management_send_show_desktop_changed(struct wl_resource *resource_, uint32_t state)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_MANAGEMENT_SHOW_DESKTOP_CHANGED, state);
}

static inline void
org_kde_plasma_window_management_send_window(struct wl_resource *resource_, uint32_t id)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_MANAGEMENT_WINDOW, id);
}

/**
 * org_kde_plasma_window - interface to control application windows
 * @set_state: set window state
 * @set_virtual_desktop: map window on a virtual desktop
 * @set_minimized_geometry: set the geometry for a taskbar entry
 * @unset_minimized_geometry: set the geometry for a taskbar entry
 * @close: close window
 *
 * Manages and control an application window.
 *
 * Only one client can bind this interface at a time.
 */
struct org_kde_plasma_window_interface {
	/**
	 * set_state - set window state
	 * @flags: bitfield of set state flags
	 * @state: bitfield of state flags
	 *
	 * Set window state.
	 *
	 * Values for state argument are described by
	 * org_kde_plasma_window_management.state and can be used together
	 * in a bitfield. The flags bitfield describes which flags are
	 * supposed to be set, the state bitfield the value for the set
	 * flags
	 */
	void (*set_state)(struct wl_client *client,
			  struct wl_resource *resource,
			  uint32_t flags,
			  uint32_t state);
	/**
	 * set_virtual_desktop - map window on a virtual desktop
	 * @number: zero based virtual desktop number
	 *
	 * Maps the window to a different virtual desktop.
	 *
	 * To show the window on all virtual desktops, call the
	 * org_kde_plasma_window.set_state request and specify a
	 * on_all_desktops state in the bitfield.
	 */
	void (*set_virtual_desktop)(struct wl_client *client,
				    struct wl_resource *resource,
				    uint32_t number);
	/**
	 * set_minimized_geometry - set the geometry for a taskbar entry
	 * @panel: (none)
	 * @x: (none)
	 * @y: (none)
	 * @width: (none)
	 * @height: (none)
	 *
	 * Sets the geometry of the taskbar entry for this window. The
	 * geometry is relative to a panel in particular.
	 */
	void (*set_minimized_geometry)(struct wl_client *client,
				       struct wl_resource *resource,
				       struct wl_resource *panel,
				       uint32_t x,
				       uint32_t y,
				       uint32_t width,
				       uint32_t height);
	/**
	 * unset_minimized_geometry - set the geometry for a taskbar
	 *	entry
	 * @panel: (none)
	 *
	 * Remove the task geometry information for a particular panel.
	 */
	void (*unset_minimized_geometry)(struct wl_client *client,
					 struct wl_resource *resource,
					 struct wl_resource *panel);
	/**
	 * close - close window
	 *
	 * Close this window.
	 */
	void (*close)(struct wl_client *client,
		      struct wl_resource *resource);
};

#define ORG_KDE_PLASMA_WINDOW_TITLE_CHANGED	0
#define ORG_KDE_PLASMA_WINDOW_APP_ID_CHANGED	1
#define ORG_KDE_PLASMA_WINDOW_STATE_CHANGED	2
#define ORG_KDE_PLASMA_WINDOW_VIRTUAL_DESKTOP_CHANGED	3
#define ORG_KDE_PLASMA_WINDOW_THEMED_ICON_NAME_CHANGED	4
#define ORG_KDE_PLASMA_WINDOW_UNMAPPED	5

#define ORG_KDE_PLASMA_WINDOW_TITLE_CHANGED_SINCE_VERSION	1
#define ORG_KDE_PLASMA_WINDOW_APP_ID_CHANGED_SINCE_VERSION	1
#define ORG_KDE_PLASMA_WINDOW_STATE_CHANGED_SINCE_VERSION	1
#define ORG_KDE_PLASMA_WINDOW_VIRTUAL_DESKTOP_CHANGED_SINCE_VERSION	1
#define ORG_KDE_PLASMA_WINDOW_THEMED_ICON_NAME_CHANGED_SINCE_VERSION	1
#define ORG_KDE_PLASMA_WINDOW_UNMAPPED_SINCE_VERSION	1

static inline void
org_kde_plasma_window_send_title_changed(struct wl_resource *resource_, const char *title)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_TITLE_CHANGED, title);
}

static inline void
org_kde_plasma_window_send_app_id_changed(struct wl_resource *resource_, const char *app_id)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_APP_ID_CHANGED, app_id);
}

static inline void
org_kde_plasma_window_send_state_changed(struct wl_resource *resource_, uint32_t flags)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_STATE_CHANGED, flags);
}

static inline void
org_kde_plasma_window_send_virtual_desktop_changed(struct wl_resource *resource_, int32_t number)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_VIRTUAL_DESKTOP_CHANGED, number);
}

static inline void
org_kde_plasma_window_send_themed_icon_name_changed(struct wl_resource *resource_, const char *name)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_THEMED_ICON_NAME_CHANGED, name);
}

static inline void
org_kde_plasma_window_send_unmapped(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ORG_KDE_PLASMA_WINDOW_UNMAPPED);
}

#ifdef  __cplusplus
}
#endif

#endif
